/*******************************************************************************
 * Copyright 2025 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#ifndef CCI_DIRECT_H
#define CCI_DIRECT_H

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

// -----------------------------------------------------------------------------------------------------------------------------
// CCI Direct
// -----------------------------------------------------------------------------------------------------------------------------

// enum of supported data types for specifying register and data types
typedef enum cci_data_type_t {
    CCI_8BIT,
    CCI_16BIT
}cci_data_type_t;


/**
 * @brief      Initializes CCI direct access for all camera sensors. This will open
 *             all cameras which are connected for cci direct access
 * *
 *
 * @return     0 on success, -1 on failure
 */
int voxl_cci_init(void);


/**
 * @brief      fetches the file descriptor for a specified cam ID so the user can
 *             do more advanced IO operations than what's presented here
 *
 * @param[in]  cam_id      The camera ID
 *
 * @return returns 0 on success, -1 on error 
 */
int voxl_cci_get_fd(int cam_id);

/**
 * @brief      fetches the number of successfully probed cameras
 *
 * @return returns the number of probed cameras
 */
uint16_t voxl_cci_get_camera_count(void);

/**
 * @brief      fetches the cameras default slave address
 *
 * @param[in]  cam_id      The camera ID
 *
 * @return returns 0 on success, -1 on error  
 */
uint16_t voxl_cci_get_slave_addr(int cam_id);


/**
 * @brief      fetches the sensor ID value
 *
 * @param[in]  cam_id      The camera ID
 *
 * @return returns 0 on success, -1 on error    
 */
uint16_t voxl_cci_get_sensor_id(int cam_id);


/**
 * @brief      fetches the sensor slot id
 *
 * @param[in]  cam_id      The camera ID
 *
 * @return returns slot id on success (>=0), -1 on error
 */
int16_t voxl_cci_get_slot_id(int cam_id);


/**
 * @brief      Read data from a register for a CCI slave device
 * *
 * @param[in]  cam_id      The camera ID
 * @param[in]  dev_addr    Slave address of the sensor
 * @param[in]  reg_addr    Register address to read from
 * @param[in]  addr_type   Address type to use for sending register address (8bit or 16bit)
 * @param[in]  read_buf    Buffer to hold the incoming data
 * @param[in]  data_type   Data type of data being read (8bit or 16bit)
 *
 * @return     returns 0 on success, -1 on error.
 */
int voxl_cci_read(int cam_id, uint8_t dev_addr, uint16_t reg_addr, cci_data_type_t addr_type, uint8_t *read_buf, cci_data_type_t data_type);


/**
 * @brief      Write data to a register for a CCI slave device
 * *
 * @param[in]  cam_id      The camera software ID
 * @param[in]  dev_addr    Slave address of the sensor
 * @param[in]  reg_addr    Register address to write to
 * @param[in]  addr_type   Address type to use for sending register address (8bit or 16bit)
 * @param[in]  write_buf   Buffer with data to write to device
 * @param[in]  data_type   Data type of data being written (8bit or 16bit)
 * 
 * @return     returns 0 on success, -1 on error.
 */
int voxl_cci_write(int cam_id, uint8_t dev_addr, uint16_t reg_addr, cci_data_type_t addr_type, uint8_t *write_buf, cci_data_type_t data_type);


/**
 * @brief      Write data to a series of registers for a CCI slave device
 * *
 * @param[in]  cam_id      The camera ID
 * @param[in]  dev_addr    Slave address of the sensor
 * @param[in]  reg_addr    Register address to access on the device
 * @param[in]  addr_type   Adress type to use for sending register address (8bit or 16bit)
 * @param[in]  reg_data    16bit data to write to register
 * @param[in]  write_size  Number of registers to be written to
 *
 * @return     returns 0 on success, -1 on error
 */
int voxl_cci_write_word_array(int cam_id, uint8_t dev_addr, uint16_t *reg_addr, cci_data_type_t addr_type, uint16_t *reg_data, uint32_t write_size);


/**
 * @brief      closes all cci camera instances
 *
 *             Please make an effort to call this before your process exits to
 *             help ensure a safe cleanup.
 *
 * @param[in]  cam_id      The camera ID
 *
 * @return     will always return 0
 */
int voxl_cci_close(void);






#ifdef __cplusplus
}
#endif

#endif  // CCI_DIRECT_H
